unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,  TeEngine, Series, ExtCtrls, TeeProcs, Chart, TeeGDIPlus, TeeTools,
  TeeDownSampling, StdCtrls;

type
  TForm1 = class(TForm)
    GroupBoxBarSeries1: TGroupBox;
    LabelBarSeries1Count: TLabel;
    GroupBoxDownSampleFun: TGroupBox;
    LabelDownSampleFunTolerance: TLabel;
    LabelDownSampleFunDisplayedPointCount: TLabel;
    GroupBoxBarSeries2: TGroupBox;
    LabelBarSeries2Count: TLabel;
    GroupBoxForm: TGroupBox;
    LabelFormClientWidth: TLabel;
    CBDraw1: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure CBDraw1Click(Sender: TObject);
  private
    { Private declarations }
    procedure ChartZoom(Sender: TObject);
    procedure ChartUndoZoom(Sender: TObject);

    procedure CreateCharts;
    procedure UpdateLabels;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses TeCanvas;

var
Chart1, Chart2: TChart;
BarSeries1, BarSeries2: TBarSeries;
DownSampleFun: TDownSamplingFunction;

procedure TForm1.FormCreate(Sender: TObject);
var
  i : Integer;
begin
  CreateCharts;

  Randomize;
  BarSeries1.Clear;
  for i := 0 to 9999 do
    BarSeries1.AddXY(i / MSecsPerDay, i mod 250);

  DownSampleFun := TDownSamplingFunction.Create(Chart1);
  BarSeries2.SetFunction(DownSampleFun);
  DownSampleFun.DownSampleMethod := dsMax;
  BarSeries2.DataSource := BarSeries1;

  DownSampleFun.Tolerance := BarSeries1.Count / ClientWidth;
  DownSampleFun.DisplayedPointCount := ClientWidth * 4;
end;

procedure TForm1.UpdateLabels;
begin
  LabelBarSeries1Count.Caption := Format('Count = %d', [BarSeries1.Count]);
  LabelDownSampleFunTolerance.Caption := Format('Tolerance = %f', [DownSampleFun.Tolerance]);
  LabelDownSampleFunDisplayedPointCount.Caption := Format('DisplayedPointCount = %d', [DownSampleFun.DisplayedPointCount]);
  LabelBarSeries2Count.Caption := Format('Count = %d', [BarSeries2.Count]);
  LabelFormClientWidth.Caption := Format('ClientWidth = %d', [ClientWidth]);
  if ClientWidth > BarSeries2.Count then
    GroupBoxForm.Color := clRed
  else
    GroupBoxForm.Color := clBtnFace;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  BarSeries2.CheckDataSource;
  UpdateLabels;
end;

procedure TForm1.ChartZoom(Sender: TObject);
begin
  if (Sender=Chart1) then
    Chart2.Axes.Bottom.SetMinMax(Chart1.Axes.Bottom.Minimum, Chart1.Axes.Bottom.Maximum)
  else
    Chart1.Axes.Bottom.SetMinMax(Chart2.Axes.Bottom.Minimum, Chart2.Axes.Bottom.Maximum);

  if CBDraw1.Checked then
    Chart1.Draw;

  BarSeries2.CheckDataSource;
  UpdateLabels;
end;

procedure TForm1.ChartUndoZoom(Sender: TObject);
begin
  Chart1.Axes.Bottom.Automatic:=true;
  Chart2.Axes.Bottom.Automatic:=true;

  if CBDraw1.Checked then
    Chart1.Draw;

  BarSeries2.CheckDataSource;
  UpdateLabels;
end;

procedure TForm1.CreateCharts;
begin
  Chart1:=TChart.Create(Self);
  with Chart1 do
  begin
    Parent:=Self;
    Height:=153;
    AllowPanning:=pmNone;
    Legend.Visible:=False;
    MarginBottom:=0;
    MarginLeft:=0;
    MarginRight:=0;
    MarginTop:=0;
    MarginUnits:=muPixels;
    OnUndoZoom:=ChartUndoZoom;
    OnZoom:=ChartZoom;
    AxisVisible:=False;
    Hover.Visible:=False;
    View3D:=False;
    Zoom.Direction:=tzdHorizontal;
    Align:=alTop;

    BarSeries1:=Chart1.AddSeries(TBarSeries) as TBarSeries;
    with BarSeries1 do
    begin
      BarPen.Visible:=False;
      Marks.Visible:=False;
      SeriesColor:=clBlue;
      BarWidthPercent:=1;
      XValues.DateTime:=True;
    end;
  end;

  Chart2:=TChart.Create(Self);
  with Chart2 do
  begin
    Parent:=Self;
    Height:=153;
    AllowPanning:=pmNone;
    Legend.Visible:=False;
    MarginBottom:=0;
    MarginLeft:=0;
    MarginRight:=0;
    MarginTop:=0;
    MarginUnits:=muPixels;
    OnUndoZoom:=ChartUndoZoom;
    OnZoom:=ChartZoom;
    AxisVisible:=False;
    Hover.Visible:=False;
    View3D:=False;
    Zoom.Direction:=tzdHorizontal;
    ZoomWheel:=pmwNormal;
    Align:=alBottom;

    BarSeries2:=Chart2.AddSeries(TBarSeries) as TBarSeries;
    with BarSeries2 do
    begin
      BarPen.Visible:=False;
      Marks.Visible:=False;
      SeriesColor:=clBlue;
      BarWidthPercent:=1;
      XValues.DateTime:=True;
    end;
  end;

  Self.OnResize:=FormResize;
end;

procedure TForm1.CBDraw1Click(Sender: TObject);
begin
  BarSeries2.CheckDataSource;
  UpdateLabels;
end;

end.
